/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) [2021] Payara Foundation and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://github.com/payara/Payara/blob/master/LICENSE.txt
 * See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * The Payara Foundation designates this particular file as subject to the "Classpath"
 * exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.weld.services;

import com.sun.enterprise.deployment.Application;
import com.sun.enterprise.deployment.BundleDescriptor;
import com.sun.enterprise.deployment.EjbBundleDescriptor;
import com.sun.enterprise.deployment.EjbDescriptor;
import junit.framework.TestCase;
import org.glassfish.deployment.common.ModuleDescriptor;
import org.glassfish.weld.WeldDeployer;
import org.jboss.weld.bootstrap.WeldBootstrap;
import org.jboss.weld.bootstrap.spi.BeanDeploymentArchive;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;

import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class JCDIServiceImplTest extends TestCase {

    @Mock
    private EjbDescriptor ejbDescriptor;

    @Mock
    private EjbBundleDescriptor ejbBundleDescriptor;

    @Mock
    private ModuleDescriptor moduleDescriptor;

    @Mock
    private BundleDescriptor bundleDescriptor;

    @Mock
    private WeldDeployer weldDeployer;

    @Mock
    private BeanDeploymentArchive beanDeploymentArchive;

    @Mock
    private WeldBootstrap bootstrap;

    @Mock
    private Application application;

    @InjectMocks
    private JCDIServiceImpl jcdiServiceImpl = new JCDIServiceImpl();

    @Test
    public void createJCDIInjectionContextWithoutNullPointerException() {
        Map<Class<?>, Object> ejbInfo = Collections.emptyMap();
        Collection<String> emptyListNames = Collections.emptyList();
        Collection<BeanDeploymentArchive> emptyListOfArchives = Collections.emptyList();
        when(ejbDescriptor.getEjbBundleDescriptor()).thenReturn(ejbBundleDescriptor);
        when(ejbBundleDescriptor.getApplication()).thenReturn(application);
        when(ejbBundleDescriptor.getModuleDescriptor()).thenReturn(moduleDescriptor);
        when(moduleDescriptor.getDescriptor()).thenReturn(bundleDescriptor);
        when(ejbDescriptor.getEjbClassName()).thenReturn("EjbName");
        when(weldDeployer.getBeanDeploymentArchiveForBundle(bundleDescriptor)).thenReturn(beanDeploymentArchive);
        when(weldDeployer.getBootstrapForApp(application)).thenReturn(bootstrap);
        when(beanDeploymentArchive.getBeanClasses()).thenReturn(emptyListNames);
        when(beanDeploymentArchive.getBeanDeploymentArchives()).thenReturn(emptyListOfArchives);

        Object obj = jcdiServiceImpl.createJCDIInjectionContext(ejbDescriptor, ejbInfo);

        assertNull(obj);
        verify(ejbDescriptor, times(2)).getEjbBundleDescriptor();
        verify(ejbBundleDescriptor, times(1)).getModuleDescriptor();
        verify(moduleDescriptor, times(1)).getDescriptor();
        verify(weldDeployer, times(1)).getBeanDeploymentArchiveForBundle(bundleDescriptor);
        verify(beanDeploymentArchive, times(1)).getBeanClasses();
        verify(beanDeploymentArchive, times(1)).getBeanDeploymentArchives();
        verify(ejbBundleDescriptor, times(1)).getApplication();
        verify(weldDeployer, times(1)).getBootstrapForApp(application);
        verify(bootstrap, times(1)).getManager(beanDeploymentArchive);
    }
}